//! Operation on String/text data
use crate::experimental::trait_based::{
    data::{
        io::{Inputs, Outputs},
        raw::DataRef,
    },
    element::{DataType, ElementSignature, PipelineElement},
};

/// Concatenate the inputs
pub struct Concatenate(pub String);
impl PipelineElement for Concatenate {
    fn runner(&self) -> fn(&Inputs) -> Outputs {
        |input| {
            if let [DataRef::String(s0), DataRef::String(s1), ..] = input.inner()[..] {
                format!("{s0}{s1}").into()
            } else {
                panic!("Invalid data passed")
            }
        }
    }

    fn signature(&self) -> ElementSignature {
        ElementSignature {
            inputs: vec![DataType::String, DataType::String],
            outputs: vec![DataType::String],
        }
    }
}

/// Turn input text to uppercase
pub struct Upper;
impl PipelineElement for Upper {
    fn runner(&self) -> fn(&Inputs) -> Outputs {
        |input| {
            if let [DataRef::String(s), ..] = input.inner()[..] {
                s.to_uppercase().into()
            } else {
                panic!("Invalid data passed")
            }
        }
    }

    fn signature(&self) -> ElementSignature {
        ElementSignature {
            inputs: vec![DataType::String],
            outputs: vec![DataType::String],
        }
    }
}

/// Turn input text to lowercase
pub struct Lower;
impl PipelineElement for Lower {
    fn runner(&self) -> fn(&Inputs) -> Outputs {
        |input| {
            if let [DataRef::String(s), ..] = input.inner()[..] {
                s.to_lowercase().into()
            } else {
                panic!("Invalid data passed")
            }
        }
    }

    fn signature(&self) -> ElementSignature {
        ElementSignature {
            inputs: vec![DataType::String],
            outputs: vec![DataType::String],
        }
    }
}
